class Messages:
    """
    This class represents list of messages, that belongs to specific guild.

    Attributes
    ----------
    messages_count : int
        Total count of guild's stored messages.
    last_message_timestamp : int
        UNIX-point when the bot received last message.
    messages : list[:class:`Message`]
        List of messages, that this guild have.
    """

    def __init__(self, data: dict) -> None:
        self.messages_count: int = int(data["messages_count"])
        self.last_message_timestamp: int = int(data["last_message_timestamp"])
        self.messages: list[Message] = list(map(Message, data["messages"]))


class Message:
    """
    This class represents message, that stored in database.

    Attributes
    ----------
    id : str
        Unique ID for this message.
    content : str
        Message's content.
    guild_id : int
        ID of guild where this message was sent.
    timestamp : int
        UNIX-point when the bot received the message.
    message_id : int
        ID of message, that this object belongs to.
    author_id : int
        ID of sender of this message.
    """

    def __init__(self, data: dict) -> None:
        self.id: str = data.get("id")
        self.content: str = data.get("content")
        self.guild_id: int = int(data.get("guild_id"))
        self.timestamp: int = int(data.get("timestamp"))
        self.message_id: int = int(data.get("message_id"))
        self.author_id: int = int(data.get("author_id"))



class APIException(Exception):
    """
    This class represents exception, that can be thrown while working with API.
    """

    ...


class RateLimited(APIException):
    """
    This class represents exception, that can be thrown when you're rate limited.
    """

    def __init__(self) -> None:
        super().__init__("You're rate limited. Try again in a few seconds.")



from typing import Union


class Settings:
    """
    This class represents guild's settings.

    Attributes
    ----------
    id : str
        Unique guild ID that stored in database.
    guild_id : int
        Discord's guild ID.
    messages_channel : int or None
        Channel, that bot will respond in.
    random_chance : int
        Chance of random respond from bot.
    random_enabled : bool
        Will bot respond with random or not.
    format_messages : bool
        Will bot format messages in his response.
    """

    def __init__(self, data: dict) -> None:
        self.id: str = data["id"]
        self.guild_id: int = int(data["guide_id"])
        self.messages_channel: Union[int, None] = data["messages_channel_id"]
        if self.messages_channel is not None:
            self.messages_channel = int(self.messages_channel)
        self.random_chance: int = int(data["random_chance"])
        self.random_enabled: bool = bool(data["random_enabled"])
        self.format_messages: bool = bool(data["format_messages"])



from requests import get

from .attachments import Attachments
from .errors import *
from .messages import Messages
from .ping import Ping
from .settings import Settings
from .statistic import Statistic


class API:
    """
    Neirogenius API wrapper.

    Attributes
    ----------
    base_url : str
        Base URL where will the requests go

    Methods
    -------
    get_statistic()
        Get statistic from the API.
    get_ping()
         Get ping from the API.
    get_messages(guild_id: int)
        Get messages from the API.
    get_attachments(guild_id: int)
        Get attachments from the API.
    get_settings(guild_id: int)
        Get settings from the API.
    """

    def __init__(self) -> None:
        self.base_url: str = "https://api.neirogenius.ru/"

    def __fetch(self, endpoint: str) -> dict:
        """
        Fetch data from the API.

        Parameters
        ----------
        endpoint : str
            The endpoint to get data from.

        Returns
        -------
        dict
            Fetched data from API.
        """
        request = get(f"{self.base_url}{endpoint}")
        data = request.json()
        if request.status_code == 429:
            raise RateLimited()
        return data

    def get_statistic(self) -> Statistic:
        """
        Get statistic from the API.

        Returns
        -------
        :class:`Statistic`
            Fetched and parsed class.
        """
        return Statistic(self.__fetch("statistic"))

    def get_ping(self) -> Ping:
        """
        Get ping from the API.

        Returns
        -------
        :class:`Ping`
            Fetched and parsed class.
        """
        return Ping(self.__fetch("ping"))

    def get_messages(self, guild_id: int) -> Messages:
        """
        Get messages from the API.

        Parameters
        ----------
        guild_id : int
            The guild ID to get messages from.

        Returns
        -------
        :class:`Messages`
            Fetched and parsed class.
        """
        return Messages(self.__fetch(f"messages/{guild_id}"))

    def get_attachments(self, guild_id: int) -> Attachments:
        """
        Get attachments from the API.

        Parameters
        ----------
        guild_id : int
            The guild ID to get attachments from.

        Returns
        -------
        :class:`Attachments`
            Fetched and parsed class.
        """
        return Attachments(self.__fetch(f"attachments/{guild_id}"))

    def get_settings(self, guild_id: int) -> Settings:
        """
        Get settings from the API.

        Parameters
        ----------
        guild_id : int
            The guild ID to get settings from.

        Returns
        -------
        :class:`Settings`
            Fetched and parsed class.
        """
        return Settings(self.__fetch(f"settings/{guild_id}"))



class Attachments:
    """
    This class represents list of attachments, that belongs to specific guild.

    Attributes
    ----------
    attachments_count : int
        Total count of guild's stored attachments.
    last_attachment_timestamp : int
        UNIX-point when the bot received last attachment.
    attachments : list[:class:`Attachment`]
        List of attachments, that this guild have.
    """

    def __init__(self, data: dict) -> None:
        self.attachments_count: int = int(data["attachments_count"])
        self.last_attachment_timestamp: int = int(data["last_attachment_timestamp"])
        self.attachments: list[Attachment] = list(map(Attachment, data["attachments"]))


class Attachment:
    """
    This class represents attachment, that stored in database.

    Attributes
    ----------
    id : str
        Unique ID for this attachment.
    url : str
        Attachment's content.
    guild_id : int
        ID of guild where this attachment was sent.
    timestamp : int
        UNIX-point when the bot received the attachment.
    message_id : int
        ID of attachment, that this object belongs to.
    author_id : int
        ID of sender of this attachment.
    """

    def __init__(self, data: dict) -> None:
        self.id: str = data.get("id")
        self.url: str = data.get("url")
        self.guild_id: int = int(data.get("guild_id"))
        self.timestamp: int = int(data.get("timestamp"))
        self.message_id: int = int(data.get("message_id"))
        self.author_id: int = int(data.get("author_id"))



class Statistic:
    """
    This class represents statistic.

    Attributes
    ----------
    guilds_count : str
        The number of guilds to which the bot has been added.
    users_count : int
        Total bot users count.
    attachments_count : int or None
        Total attachments count.
    messages_count : int
        Total messages count.
    guilds : list[:class:`Guild`]
        List of :class:`Guild` instances, that bot was added to.
    """

    def __init__(self, data: dict) -> None:
        self.guilds_count: int = int(data["guilds_count"])
        self.users_count: int = int(data["users_count"])
        self.attachments_count: int = int(data["attachments_count"])
        self.messages_count: int = int(data["messages_count"])
        self.guilds: list[Guild] = list(map(Guild, data["guilds"]))


class Guild:
    """
    This class represents guild from statistic.

    Attributes
    ----------
    name : str
        Name of guild.
    members_count : int
        Count of members in this guild.
    """

    def __init__(self, data: dict) -> None:
        self.name: str = data["name"]
        self.members_count: int = int(data["members_count"])



from setuptools import setup

with open("README.md", "r", encoding="utf-8") as readme_file:
    long_description = readme_file.read()

setup(
    name="neirogenius",
    version="1.1.0",
    license="Apache License 2.0",
    description="Simple wrapper for the Neirogenius API",
    author="itsSourCream",
    author_email="me@itssourcream.space",
    packages=["neirogenius"],
    install_requires=["requests"],
    url="https://github.com/itsSourCream/NeirogeniusPY",
    long_description=long_description,
    long_description_content_type="text/markdown",
)



class Ping:
    """
    This class represents ping, that belongs to bot.

    Attributes
    ----------
    global_ping : int
        Global bot ping.
    global_formatted_ping : int
        Formatted version of `global_ping`.
    shards : list[:class:`Shard`]
        List of shards, that the bot have.
    """

    def __init__(self, data: dict) -> None:
        self.global_ping: int = int(data["global_ping"])
        self.global_formatted_ping: str = data["global_formatted_ping"]
        self.shards: list[Shard] = list(map(Shard, data["shards"]))


class Shard:
    """
    This class represents shard, that belongs to bot.

    Attributes
    ----------
    shard_ping : int
        Shard's ping.
    shard_formatted_ping : str
        Formatted version of `shard_ping`.
    """

    def __init__(self, data: dict) -> None:
        self.shard_ping: int = int(data["shard_ping"])
        self.shard_formatted_ping: str = data["shard_formatted_ping"]
